from typing import Optional
import pytest

from gcs_migration.typing.transfer_documents import AccessDocumentDict, RoleTypeName
from gcs_migration.models import Role

from gcs_migration.check.role import (
    RoleCheck,
    RO01,
    RO02,
    RO03,
    RO04,
    RO05,
    RO06,
)


def test_check_ro01(access_role_document: AccessDocumentDict) -> None:
    ro01 = RO01()
    checker = RoleCheck()
    role = Role()
    role.access_document = access_role_document
    role.principal = None

    assert ro01.is_ok(None)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, RO01), res)) == []

    assert not ro01.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO01), res)) != []

    role.principal = "foo"
    assert ro01.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO01), res)) == []


def test_check_ro02(access_role_document: AccessDocumentDict) -> None:
    ro02 = RO02()
    checker = RoleCheck()
    role = Role()
    role.access_document = access_role_document
    role.principal = None

    assert ro02.is_ok(None)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, RO02), res)) == []

    assert ro02.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO02), res)) == []

    role.principal = "foo"
    assert not ro02.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO02), res)) != []

    role.principal = "30907d78-d009-46b7-9971-bc9de259d953"
    assert ro02.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO02), res)) == []


def test_check_ro03(access_role_document: AccessDocumentDict) -> None:
    ro03 = RO03()
    checker = RoleCheck()
    role = Role()
    role.access_document = access_role_document
    role.principal_type = None

    assert ro03.is_ok(None)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, RO03), res)) == []

    assert not ro03.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO03), res)) != []

    role.principal_type = "identity"
    assert ro03.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO03), res)) == []


def test_check_ro04(access_role_document: AccessDocumentDict) -> None:
    ro04 = RO04()
    checker = RoleCheck()
    role = Role()
    role.access_document = access_role_document
    role.principal_type = "identity"

    assert ro04.is_ok(None)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, RO04), res)) == []

    assert ro04.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO04), res)) == []

    role.principal_type = "group"
    assert ro04.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO04), res)) == []

    role.principal_type = "GROUP"  # type: ignore
    assert not ro04.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO04), res)) != []


def test_check_ro05(access_role_document: AccessDocumentDict) -> None:
    ro05 = RO05()
    checker = RoleCheck()
    role = Role()
    role.access_document = access_role_document
    role.role = None

    assert ro05.is_ok(None)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, RO05), res)) == []

    assert not ro05.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO05), res)) != []

    role.role = "administrator"
    assert ro05.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO05), res)) == []


@pytest.mark.parametrize(
    "role_name",
    ["administrator", "access_manager", "activity_manager", "activity_monitor"],
)
def test_check_ro06(
    access_role_document: AccessDocumentDict, role_name: Optional[RoleTypeName]
) -> None:
    ro06 = RO06()
    checker = RoleCheck()
    role = Role()
    role.access_document = access_role_document
    role.role = None

    assert ro06.is_ok(None)

    res = checker.check(None)
    assert list(filter(lambda x: isinstance(x, RO06), res)) == []

    assert ro06.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO06), res)) == []

    role.role = role_name
    assert ro06.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO06), res)) == []

    role.role = "frobnicator"  # type: ignore
    assert not ro06.is_ok(role)
    res = checker.check(role)
    assert list(filter(lambda x: isinstance(x, RO06), res)) != []
