import json
import os
import pathlib
from typing import Any, Generator, List, Iterable, Optional, Tuple
from unittest import mock

import pytest
from click.testing import CliRunner

from gcs_migration.commands.identity_mapping_custom import identity_mapping_custom
from gcs_migration.models import MigrationPlan
from gcs_migration.typing.gcs_documents import IdentityMappingDict


@pytest.fixture
def mock_map_identities_environment(
    mock_get_transfer_api: Any,
    mock_get_auth_api: Any,
    migration_plan_cmd_create: MigrationPlan,
    authorization_response_success: Tuple[Optional[str], str, int],
    tmp_path: pathlib.Path,
) -> Generator[MigrationPlan, None, None]:
    with mock.patch.object(MigrationPlan, "load") as mock_migration_load:
        mock_migration_load.return_value = migration_plan_cmd_create
        migration_plan_cmd_create.root = tmp_path
        os.mkdir(tmp_path / "migration_plan")
        yield migration_plan_cmd_create


def _mapping_for_domains(domains: Iterable[str]) -> List[IdentityMappingDict]:
    escaped_domains = [d.replace(".", "\\.") for d in domains]
    return [
        {
            "DATA_TYPE": "expression_identity_mapping#1.0.0",
            "mappings": [
                {
                    "source": "{username}",
                    "match": f"(.*)@({'|'.join(escaped_domains)})",
                    "output": "{0}",
                },
            ],
        }
    ]


@pytest.fixture(params=[(None, False), (["other.edu"], False), (["other.edu"], True)])
def mapping_file(
    request: Any, tmp_path: pathlib.Path
) -> Optional[Tuple[Optional[str], List[IdentityMappingDict]]]:
    domains, create_file = request.param

    if not domains:
        return None

    mapping = _mapping_for_domains(domains)

    mapfile: Optional[str] = None

    if create_file:
        mapfile = str(tmp_path / "mapping.json")
        with open(mapfile, "w") as f:
            json.dump(mapping[0], f)

    return (mapfile, mapping)


@pytest.mark.parametrize(
    "domains",
    [["example.org", "example.org", "example.edu"], ["example.org", "example.edu"]],
)
def test_identity_mapping_custom(
    mock_map_identities_environment: MigrationPlan,
    domains: List[str],
    mapping_file: Optional[Tuple[Optional[str], Optional[List[IdentityMappingDict]]]],
) -> None:
    runner = CliRunner()
    args: List[str] = []
    migration_plan = mock_map_identities_environment

    for d in domains:
        args.extend(["-d", d])

    if mapping_file:
        mapping_file_path, mappings = mapping_file
    else:
        mapping_file_path, mappings = None, None

    if mapping_file_path:
        args.extend(["-i", mapping_file_path])
    elif mappings:
        for m in mappings:
            args.extend(["-i", json.dumps(m)])

    result = runner.invoke(identity_mapping_custom, args, catch_exceptions=False)
    assert result.exit_code == 0

    assert migration_plan.storage_gateway is not None
    assert migration_plan.storage_gateway.allowed_domains is not None

    assert set(domains) == set(migration_plan.storage_gateway.allowed_domains)

    if mappings:
        expected_mappings = mappings
    else:
        expected_mappings = _mapping_for_domains(domains)

    assert migration_plan.storage_gateway.identity_mappings is not None

    for m in expected_mappings:
        assert m in migration_plan.storage_gateway.identity_mappings

    for m in migration_plan.storage_gateway.identity_mappings:
        assert m in expected_mappings
